package complex;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotSame;
import static org.junit.Assert.assertSame;

import org.junit.Before;
import org.junit.Test;

public class ComplexTest {

	private final static double DELTA = 0.00001;

	private Complex c1;
	private Complex c2;

	@Before
	public void setUp() {
		c1 = new Complex(1.2, 4.3);
		c2 = new Complex(2.1, 3.4);
	}

	@Test
	public void copy() {
		Complex c = new Complex(c1);
		assertNotSame(c1, c);
		assertEquals(1.2, c.real());
		assertEquals(4.3, c.imag());
	}

	@Test
	public void add() {
		Complex c = c1.add(c2);
		assertSame(c1, c);
		assertEquals(3.3, c.real(), DELTA);
		assertEquals(7.7, c.imag(), DELTA);
	}

	@Test
	public void sub() {
		Complex c = c1.sub(c2);
		assertSame(c1, c);
		assertEquals(-0.9, c.real(), DELTA);
		assertEquals(0.9, c.imag(), DELTA);
	}

	@Test
	public void mul() {
		Complex c = c1.mul(c2);
		assertSame(c1, c);
		assertEquals(-12.1, c.real(), DELTA);
		assertEquals(13.11, c.imag(), DELTA);
	}

	@Test
	public void div() {
		Complex c = c1.div(c2);
		assertSame(c1, c);
		assertEquals(1.07326, c.real(), DELTA);
		assertEquals(0.30996, c.imag(), DELTA);
	}

	@Test
	public void square() {
		Complex c = c1.square();
		assertSame(c1, c);
		assertEquals(-17.05, c.real(), DELTA);
		assertEquals(10.32, c.imag(), DELTA);
	}

	@Test
	public void cube() {
		Complex c = c1.cube();
		assertSame(c1, c);
		assertEquals(-64.836, c.real(), DELTA);
		assertEquals(-60.931, c.imag(), DELTA);
	}

	@Test
	public void modsq() {
		assertEquals(15.97, c2.modsq(), DELTA);
	}

	@Test
	public void stringRepresentation() {
		assertEquals("1.2 + 4.3i", c1.toString());
	}
}
